/*

pilotmain.cpp

eNeRgy - A free music player for Sony Clie NR70

Copyright (C) 2002 Tim Toner

This software is licensed under the GNU GPL. See:
http://www.gnu.org/licenses/gpl.html

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Special thanks to Rich McNeary, creator of a C++ code skeleton for
PalmOS programming, which was used in this project. See:
http://www.home.eznet.net/~rlmsmw/palmos/palmos.htm

This program was built using:
prc-tools 2.1
Palm SDK 4.0
Sony Clie SDK 3.0

*/


///////////////////////////////////////////////////////////////////////////////
// System headers

#include <PalmOS.h>
#include <SonyCLIE.h>


///////////////////////////////////////////////////////////////////////////////
// Developer created headers

#include "globals.h"
#include "resource.h"
#include "pilotmain.h"


// ****************************************************************************
// The function that the handheld looks for when the app is launched
// ****************************************************************************
UInt32 PilotMain(UInt16 uiLaunchCommand, MemPtr cmdPBP, UInt16 uiLaunchFlags)
{
   Err error = 0;



   // Check the OS version



   // Normal launch


   if(uiLaunchCommand == sysAppLaunchCmdNormalLaunch)
   {
      error = Start_Application();
      if(!error)
      {
         Event_Loop();
         Stop_Application();
      }
   }


	return(0);
}


// ****************************************************************************
// Initialize the application when it starts.
// ****************************************************************************
static Err Start_Application(void)
{
   UInt32 uiStartFormId = FRM_MAIN;
   Err error;

   // Open the databases
   // Check database versions

   // Check for Sony Clie features
   error = LoadSony();
   if (error) return(error);


   // Load Clie hi-res library
//   error = LoadLibHR();
//   if (error) return(error);


   // Load Clie Silk library
//   error = LoadLibSilk();
//   if (error) return(error);


   // Load Clie Memory Stick Audio library
   error = LoadLibMsa();
   if (error) return(error);


   // Load the starting form
   FrmGotoForm(uiStartFormId);
   return(errNone);
}


// ****************************************************************************
// Stop the application.
// ****************************************************************************
static void Stop_Application(void)
{
   // Close all databases
   // Close all forms

   FrmCloseAllForms();

	return;
}


// ****************************************************************************
// Sets the active form and funnels events to the appropriate form function.
// ****************************************************************************
static Boolean Application_Handle_Event(EventPtr event)
{
   FormPtr ptrLoadForm;
   UInt32 uiFormId;
   Boolean bHandled = false;

   if(event->eType == frmLoadEvent)
   {
      // Load and make active the event form
      uiFormId = event->data.frmLoad.formID;
      ptrLoadForm = FrmInitForm(uiFormId);
      FrmSetActiveForm(ptrLoadForm);

      // Set the form's event handler
      switch(uiFormId)
      {
         //Table
         case FRM_MAIN:
            FrmSetEventHandler(ptrLoadForm, Main_Form_Handle_Event);
            bHandled = true;
            break;
      }
   }

	return(bHandled);
}


// ****************************************************************************
// The event loop!
// ****************************************************************************
static void Event_Loop(void)
{
   EventType event;
   UInt16 error;

   do{
      EvtGetEvent(&event, evtWaitForever);
      if (! SysHandleEvent(&event))
         if (! MenuHandleEvent(0, &event, &error))
            if (! Application_Handle_Event(&event))
               FrmDispatchEvent(&event);
   } while (event.eType != appStopEvent);
}


// ****************************************************************************
// check for Sony features
// ****************************************************************************
static Err LoadSony()
{
   Err error;

   error = FtrGet(sonySysFtrCreator, sonySysFtrNumSysInfoP, (UInt32*)&sonySysFtrSysInfoP);
   if (error)
   {
      /* Not CLIE: maybe not available */
      FrmCustomAlert(ALT_PROBLEM, "finding", "features", "101");
      return error;
   }

   return errNone;
}


// ****************************************************************************
// load Sony High Resolution library
// ****************************************************************************
static Err LoadLibHR()
{
   Err error;
   UInt16 refNum;

   if (!(sonySysFtrSysInfoP->libr & sonySysFtrSysInfoLibrHR))
   {
      FrmCustomAlert(ALT_PROBLEM, "finding", "High Resolution features", "201");
      return sysErrLibNotFound;
   }

   error = SysLibFind(sonySysLibNameHR, &refNum);
   if (error)
   {
      FrmCustomAlert(ALT_PROBLEM, "finding", "High Resolution library", "202");
      return error;
   }

   LibrHR = refNum;
   return errNone;
}


// ****************************************************************************
// load Sony Silk (software graffiti) library
// ****************************************************************************
static Err LoadLibSilk()
{
   Err error = 0;
   UInt16 refNum;

   if (!(sonySysFtrSysInfoP->extn & sonySysFtrSysInfoExtnSilk))
   {
      //FrmCustomAlert(ALT_PROBLEM, "finding", "Silk features, "401");
      return sysErrLibNotFound;
   }

   if (!(sonySysFtrSysInfoP->libr & sonySysFtrSysInfoLibrSilk))
   {
      //FrmCustomAlert(ALT_PROBLEM, "finding", "Silk library", "402");
      return sysErrLibNotFound;
   }

   error = SysLibFind(sonySysLibNameSilk, &refNum);
   if (error == sysErrLibNotFound) error = SysLibLoad('libr', sonySysFileCSilkLib, &refNum );
   if (error)
   {
      //FrmCustomAlert(ALT_PROBLEM, "finding", "Silk library", "403");
      return error;
   }

   error = SilkLibOpen(refNum);
   {
      //FrmCustomAlert(ALT_PROBLEM, "opening", "Silk library", "404");
      return error;
   }

   LibrSilk = refNum;
   SilkLibEnableResize(LibrSilk);
   return errNone;
}


// ****************************************************************************
// load Sony Memory Stick Audio library
// ****************************************************************************
static Err LoadLibMsa()
{
   SonySysFtrSysInfoP sonySysFtrSysInfoP;
   Err error = 0;
   UInt16 refNum;

   error = SysLibFind(sonySysLibNameMsa, &refNum);
   if (error)
   {
      // can't find, try loading it
      error = SysLibLoad('libr', sonySysFileCMsaLib, &refNum);
      if (error)
      {
         FrmCustomAlert(ALT_PROBLEM, "finding", "Memory Stick Audio library", "301");
         return error;
      }
   }

   error = MsaLibOpen(refNum, msaLibOpenModeAlbum);
   if (error && error != msaErrAlreadyOpen)
   {
      FrmCustomAlert(ALT_PROBLEM, "opening", "Memory Stick Audio library", "302");
      return error;
   }

   LibrMsa = refNum;

   error = MsaGetAlbum(LibrMsa, &albumRefNum, &dummy);
   if (error)
   {
      FrmCustomAlert(ALT_PROBLEM, "getting", "album.", "303");
      return error;
   }

   return errNone;
}


// ****************************************************************************
// Sony Memory Stick Audio library routines
// ****************************************************************************
extern void SongDisplay()
{
   char tmpstring[20];
   char * cp;
   Err error;

   UInt32 albumIterator=albumIteratorStart;

   MsaGetPBStatus(LibrMsa, &songStatus);
   MsaGetPBList(LibrMsa, NULL, &totalSongs);
   MsaGetPBMode(LibrMsa, &songMode);

   WinSetForeColorRGB(&white, NULL);
   RectangleType r0 = {{0, 35},{160, 10}};
   WinPaintRectangle(&r0 , 0);

   StrPrintF(tmpstring, "Track: %d/%d", songStatus.currentpblistindex, totalSongs);
   WinSetForeColorRGB(&black, NULL);
   WinDrawChars(tmpstring, StrLen(tmpstring), 5, 35 );

   // volume bar
   MsaOutGetVolume(LibrMsa, &LVol, &RVol);
   int V = (LVol+RVol)/2;

   WinSetForeColorRGB(&grey3, NULL);
   RectangleType r1 = {{124, 69},{31, 2}};
   WinPaintRectangle(&r1 , 0);

   WinSetForeColorRGB(&black, NULL);
   RectangleType r2 = {{124, 69},{V, 2}};
   WinPaintRectangle(&r2 , 0);

}


extern void SongPlay()
{
   MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeySet);
   MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeyRelease);
   SongDisplay();
}


extern void SongPause()
{
   if (songStatus.status == msa_STOPSTATUS)
   {
      MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeyRelease);
   }
   else
   {
      MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeyRelease);
   }

   SongDisplay();

}


extern void SongStop()
{
   MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeySet);
   MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeyRelease);

   MsaGetPBStatus(LibrMsa, &songStatus);
   songStatus.currentSU = 0;
   MsaSetPBStatus(LibrMsa, &songStatus);

   SongDisplay();
}


extern void SongEject()
{
   MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeySet);
   MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeyRelease);

   EventType evtStop;
   MemSet(&evtStop, sizeof(EventType), 0);
   evtStop.eType = appStopEvent;
   EvtAddEventToQueue (&evtStop);
}


extern void SongPrev()
{
   bool play = false;
   MsaGetPBStatus(LibrMsa, &songStatus);

   if (songStatus.currentpblistindex <= 1)
   {
      if (songStatus.status == msa_PLAYSTATUS)
      {
         MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeySet);
         MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeyRelease);
         play = true;
      }

      songStatus.currentpblistindex = totalSongs;
      songStatus.currentSU = 0;
      MsaSetPBStatus(LibrMsa, &songStatus);

      if (play)
      {
         MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeySet);
         MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeyRelease);
      }
   }
   else
   {
      MsaSetControlKey(LibrMsa, msaControlkeyFR, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyFR, msaControlKeyRelease);
   }

   SongDisplay();
}


extern void SongNext()
{
   bool play;
   MsaGetPBStatus(LibrMsa, &songStatus);

   if (songStatus.currentpblistindex >= totalSongs)
   {
      if(songStatus.status == msa_PLAYSTATUS)
      {
         MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeySet);
         MsaSetControlKey(LibrMsa, msaControlkeyStop, msaControlKeyRelease);
         play = true;
      }

      songStatus.currentpblistindex = 1;
      songStatus.currentSU = 0;
      MsaSetPBStatus(LibrMsa, &songStatus);

      if (play)
      {
         MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeySet);
         MsaSetControlKey(LibrMsa, msaControlkeyPlay, msaControlKeyRelease);
      }

   }
   else
   {
      MsaSetControlKey(LibrMsa, msaControlkeyFF, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyFF, msaControlKeyRelease);
   }

   SongDisplay();
}

extern void VolUp()
{
   for (int i=0; i<4; i++)
   {
      MsaSetControlKey(LibrMsa, msaControlkeyVolp, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyVolp, msaControlKeyRelease);
   }

   SongDisplay();
}

extern void VolDn()
{
   for (int i=0; i<4; i++)
   {
      MsaSetControlKey(LibrMsa, msaControlkeyVolm, msaControlKeySet);
      MsaSetControlKey(LibrMsa, msaControlkeyVolm, msaControlKeyRelease);
   }

   SongDisplay();
}











