/*

pilotmain.h

eNeRgy - A free music player for Sony Clie NR70

Copyright (C) 2002 Tim Toner

This software is licensed under the GNU GPL. See:
http://www.gnu.org/licenses/gpl.html

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Special thanks to Rich McNeary, creator of a C++ code skeleton for
PalmOS programming, which was used in this project. See:
http://www.home.eznet.net/~rlmsmw/palmos/palmos.htm

This program was built using:
prc-tools 2.1
Palm SDK 4.0
Sony Clie SDK 3.0

*/
#define _PILOTMAIN_H_


/* Globals */

MemPtr    mp;        // a memory pointer

SonySysFtrSysInfoP sonySysFtrSysInfoP = NULL;
UInt16 LibrHR;       // resource handles to Sony Clie libraries
UInt16 LibrSilk;
UInt16 LibrMsa;

UInt32 dummy;        // for Msa functions
UInt16 albumRefNum;
UInt32 albumIterator;

AlbumInfoType albumInfo;
MsaTrackInfo songInfo;
char albumPath[80];
MsaPBMode songMode;
MsaPBStatus songStatus;
MsaPBList songList;
UInt16 totalSongs;
UInt16 LVol, RVol;

int t; // ticks

RGBColorType black = { 0, 0, 0, 0 };
RGBColorType white = { 0, 255, 255, 255 };
RGBColorType grey1 = { 0, 64, 64, 64 };
RGBColorType grey2 = { 0, 128, 128, 128 };
RGBColorType grey3 = { 0, 196, 196, 196 };

///////////////////////////////////////////////////////////////////////////////
// Functions

/***********************************************************************************
*                                                                                  *
* PilotMain                                                                        *
*                                                                                  *
* UInt32 PilotMain(UInt16 uiLaunchCommand, MemPtr cmdPBP, UInt16 uiLaunchFlags)    *
*                                                                                  *
* Parameters:                                                                      *
* -> uiLaunchCommand describes the type of launch                                  *
* -> cmdPBP parameter block pointer                                                *
* -> uiLaunchFlags describe various launch information                             *
*                                                                                  *
* Returns:                                                                         *
* -> If PilotMain exits normally it will return 0.                                 *
*                                                                                  *
***********************************************************************************/
UInt32 PilotMain(UInt16 uiLaunchCommand, MemPtr cmdPBP, UInt16 uiLaunchFlags);


/***********************************************************************************
*                                                                                  *
* Start_Application                                                                *
*                                                                                  *
* static Err Start_Application(void);                                              *
*                                                                                  *
* Parameters:                                                                      *
* -> none                                                                          *
*                                                                                  *
* Returns:                                                                         *
* Any errors generated by the function or 0 if there were no errors.               *
*                                                                                  *
* Some things that may be done here are opening databases and setting the first    *
* form to display.                                                                 *
*                                                                                  *
***********************************************************************************/
static Err Start_Application(void);


/***********************************************************************************
*                                                                                  *
* Stop_Application                                                                 *
*                                                                                  *
* static void Stop_Application(void);                                              *
*                                                                                  *
* Parameters:                                                                      *
* -> none                                                                          *
*                                                                                  *
* Returns:                                                                         *
* Nothing.                                                                         *
*                                                                                  *
* Some things that may be done here are closing databases and closing forms.       *
*                                                                                  *
***********************************************************************************/
static void Stop_Application(void);


/***********************************************************************************
*                                                                                  *
* Application_Handle_Event                                                         *
*                                                                                  *
* static Boolean Application_Handle_Event(EventPtr event);                         *
*                                                                                  *
* Parameters:                                                                      *
* -> event is the event to be worked with or passed to a form handler.             *
*                                                                                  *
* Returns:                                                                         *
* True is the event was handled, false if it was not.                              *
*                                                                                  *
* Load and make active the appropriate form and set the form event handler         *
* callback function for each form..                                                *
*                                                                                  *
***********************************************************************************/
static Boolean Application_Handle_Event(EventPtr event);


/***********************************************************************************
*                                                                                  *
* Event_Loop                                                                       *
*                                                                                  *
* static void Event_Loop(void);                                                    *
*                                                                                  *
* Parameters:                                                                      *
* -> None                                                                          *
*                                                                                  *
* Returns:                                                                         *
* Nothing                                                                          *
*                                                                                  *
* Chcks for and routes events to the appropriate handler. Events are sent to the   *
* handlers in the following order: system, menu, application while the event is    *
* not an appStopEvent.                                                             *
*                                                                                  *
***********************************************************************************/
static void Event_Loop(void);


static Err LoadSony();
static Err LoadLibHR();
static Err LoadLibSilk();
static Err LoadLibMsa();


/* External Functions */
extern Boolean Main_Form_Handle_Event(EventPtr event) FORM_SECTION;
extern void SongDisplay();
extern void SongPlay();
extern void SongPause();
extern void SongStop();
extern void SongEject();
extern void SongFF();
extern void SongRev();
extern void SongPrev();
extern void SongNext();
extern void VolUp();
extern void VolDn();





